@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -41,7 +41,6 @@
 #include "DlAbortEx.h"
 #include "message.h"
 #include "Peer.h"
-#include "StringFormat.h"
 #include "PieceStorage.h"
 #include "a2functional.h"
 
@@ -49,15 +48,34 @@ namespace aria2 {
 
 const std::string BtBitfieldMessage::NAME("bitfield");
 
-void BtBitfieldMessage::setBitfield(const unsigned char* bitfield, size_t bitfieldLength) {
-  if(this->bitfield == bitfield) {
+BtBitfieldMessage::BtBitfieldMessage():SimpleBtMessage(ID, NAME),
+                                       bitfield_(0),
+                                       bitfieldLength_(0)
+{}
+
+BtBitfieldMessage::BtBitfieldMessage
+(const unsigned char* bitfield, size_t bitfieldLength):
+  SimpleBtMessage(ID, NAME),
+  bitfield_(0),
+  bitfieldLength_(0)
+{
+  setBitfield(bitfield, bitfieldLength);
+}
+
+BtBitfieldMessage::~BtBitfieldMessage()
+{
+  delete [] bitfield_;
+}
+
+void BtBitfieldMessage::setBitfield
+(const unsigned char* bitfield, size_t bitfieldLength) {
+  if(bitfield_ == bitfield) {
     return;
   }
-  delete [] this->bitfield;
-
-  this->bitfieldLength = bitfieldLength;
-  this->bitfield = new unsigned char[this->bitfieldLength];
-  memcpy(this->bitfield, bitfield, this->bitfieldLength);
+  delete [] bitfield_;
+  bitfieldLength_ = bitfieldLength;
+  bitfield_ = new unsigned char[bitfieldLength_];
+  memcpy(bitfield_, bitfield, bitfieldLength_);
 }
 
 BtBitfieldMessageHandle
@@ -71,39 +89,37 @@ BtBitfieldMessage::create(const unsigned char* data, size_t dataLength)
 }
 
 void BtBitfieldMessage::doReceivedAction() {
-  if(_metadataGetMode) {
+  if(isMetadataGetMode()) {
     return;
   }
-  pieceStorage->updatePieceStats(bitfield, bitfieldLength, peer->getBitfield());
-  peer->setBitfield(bitfield, bitfieldLength);
-  if(peer->isSeeder() && pieceStorage->downloadFinished()) {
+  getPieceStorage()->updatePieceStats(bitfield_, bitfieldLength_,
+                                      getPeer()->getBitfield());
+  getPeer()->setBitfield(bitfield_, bitfieldLength_);
+  if(getPeer()->isSeeder() && getPieceStorage()->downloadFinished()) {
     throw DL_ABORT_EX(MSG_GOOD_BYE_SEEDER);
   }
 }
 
-const unsigned char* BtBitfieldMessage::getMessage() {
-  if(!msg) {
-    /**
-     * len --- 1+bitfieldLength, 4bytes
-     * id --- 5, 1byte
-     * bitfield --- bitfield, len bytes
-     * total: 5+len bytes
-     */
-    msgLength = 5+bitfieldLength;
-    msg = new unsigned char[msgLength];
-    bittorrent::createPeerMessageString(msg, msgLength, 1+bitfieldLength, ID);
-    memcpy(msg+5, bitfield, bitfieldLength);
-  }
+unsigned char* BtBitfieldMessage::createMessage() {
+  /**
+   * len --- 1+bitfieldLength, 4bytes
+   * id --- 5, 1byte
+   * bitfield --- bitfield, len bytes
+   * total: 5+len bytes
+   */
+  const size_t msgLength = 5+bitfieldLength_;
+  unsigned char* msg = new unsigned char[msgLength];
+  bittorrent::createPeerMessageString(msg, msgLength, 1+bitfieldLength_, ID);
+  memcpy(msg+5, bitfield_, bitfieldLength_);
   return msg;
 }
 
 size_t BtBitfieldMessage::getMessageLength() {
-  getMessage();
-  return msgLength;
+  return 5+bitfieldLength_;
 }
 
 std::string BtBitfieldMessage::toString() const {
-  return strconcat(NAME, " ", util::toHex(bitfield, bitfieldLength));
+  return strconcat(NAME, " ", util::toHex(bitfield_, bitfieldLength_));
 }
 
 } // namespace aria2
