@@ -34,9 +34,8 @@
 /* copyright --> */
 #include "DHTRoutingTableSerializer.h"
 
-#include <cerrno>
 #include <cstring>
-#include <ostream>
+#include <cstdio>
 
 #include "DHTNode.h"
 #include "DlAbortEx.h"
@@ -44,95 +43,115 @@
 #include "bittorrent_helper.h"
 #include "Logger.h"
 #include "a2netcompat.h"
-#include "StringFormat.h"
 #include "util.h"
+#include "TimeA2.h"
+#include "fmt.h"
+#include "File.h"
+#include "LogFactory.h"
+#include "BufferedFile.h"
 
 namespace aria2 {
 
-DHTRoutingTableSerializer::DHTRoutingTableSerializer() {}
+DHTRoutingTableSerializer::DHTRoutingTableSerializer(int family):
+  family_(family) {}
 
 DHTRoutingTableSerializer::~DHTRoutingTableSerializer() {}
 
 void DHTRoutingTableSerializer::setLocalNode
 (const SharedHandle<DHTNode>& localNode)
 {
-  _localNode = localNode;
+  localNode_ = localNode;
 }
 
 void DHTRoutingTableSerializer::setNodes
 (const std::vector<SharedHandle<DHTNode> >& nodes)
 {
-  _nodes = nodes;
+  nodes_ = nodes;
 }
 
-void DHTRoutingTableSerializer::serialize(std::ostream& o)
+#define WRITE_CHECK(fp, ptr, count)                                     \
+  if(fp.write((ptr), (count)) != (count)) {                             \
+    throw DL_ABORT_EX(fmt("Failed to save DHT routing table to %s.",    \
+                          filename.c_str()));                           \
+  }
+
+void DHTRoutingTableSerializer::serialize(const std::string& filename)
 {
+  A2_LOG_INFO(fmt("Saving DHT routing table to %s.", filename.c_str()));
+  std::string filenameTemp = filename+"__temp";
+  BufferedFile fp(filenameTemp, BufferedFile::WRITE);
+  if(!fp) {
+    throw DL_ABORT_EX(fmt("Failed to save DHT routing table to %s.",
+                          filename.c_str()));
+  }
   char header[8];
   memset(header, 0, sizeof(header));
   // magic
-  header[0] = 0xa1;
-  header[1] = 0xa2;
+  header[0] = 0xa1u;
+  header[1] = 0xa2u;
   // format ID
-  header[2] = 0x02;
+  header[2] = 0x02u;
   // version
   header[6] = 0;
-  header[7] = 0x03;
+  header[7] = 0x03u;
   
-  char zero[16];
+  char zero[18];
   memset(zero, 0, sizeof(zero));
 
-  o.write(header, 8);
+  WRITE_CHECK(fp, header, 8);
   // write save date
   uint64_t ntime = hton64(Time().getTime());
-  o.write(reinterpret_cast<const char*>(&ntime), sizeof(ntime));
+  WRITE_CHECK(fp, &ntime, sizeof(ntime));
 
   // localnode
   // 8bytes reserved
-  o.write(zero, 8);
+  WRITE_CHECK(fp, zero, 8);
   // 20bytes localnode ID
-  o.write(reinterpret_cast<const char*>(_localNode->getID()), DHT_ID_LENGTH);
+  WRITE_CHECK(fp, localNode_->getID(), DHT_ID_LENGTH);
   // 4bytes reserved
-  o.write(zero, 4);
+  WRITE_CHECK(fp, zero, 4);
 
   // number of nodes
-  uint32_t numNodes = htonl(_nodes.size());
-  o.write(reinterpret_cast<const char*>(&numNodes), sizeof(uint32_t));
+  uint32_t numNodes = htonl(nodes_.size());
+  WRITE_CHECK(fp, &numNodes, sizeof(uint32_t));
   // 4bytes reserved
-  o.write(zero, 4);
+  WRITE_CHECK(fp, zero, 4);
 
+  const int clen = bittorrent::getCompactLength(family_);
   // nodes
-  for(std::vector<SharedHandle<DHTNode> >::const_iterator i = _nodes.begin(),
-        eoi = _nodes.end(); i != eoi; ++i) {
+  for(std::vector<SharedHandle<DHTNode> >::const_iterator i = nodes_.begin(),
+        eoi = nodes_.end(); i != eoi; ++i) {
     const SharedHandle<DHTNode>& node = *i;
-    // Currently, only IPv4 address and IPv4-mapped address are saved.
-    // 6bytes: write IP address + port in Compact IP-address/port info form.
-    unsigned char compactPeer[6];
-    if(!bittorrent::createcompact
-       (compactPeer, node->getIPAddress(), node->getPort())) {
-      memset(compactPeer, 0, 6);
+    // Write IP address + port in Compact IP-address/port info form.
+    unsigned char compactPeer[COMPACT_LEN_IPV6];
+    int compactlen = bittorrent::packcompact
+      (compactPeer, node->getIPAddress(), node->getPort());
+    if(compactlen != clen) {
+      memset(compactPeer, 0, clen);
     }
+    uint8_t clen1 = clen;
     // 1byte compact peer format length
-    o << static_cast<uint8_t>(sizeof(compactPeer));
+    WRITE_CHECK(fp, &clen1, sizeof(clen1));
     // 7bytes reserved
-    o.write(zero, 7);
-    // 6 bytes compact peer
-    o.write(reinterpret_cast<const char*>(compactPeer), 6);
-    // 2bytes reserved
-    o.write(zero, 2);
-    // 16bytes reserved
-    o.write(zero, 16);
+    WRITE_CHECK(fp, zero, 7);
+    // clen bytes compact peer
+    WRITE_CHECK(fp, compactPeer, static_cast<size_t>(clen));
+    // 24-clen bytes reserved
+    WRITE_CHECK(fp, zero, 24-clen);
     // 20bytes: node ID
-    o.write(reinterpret_cast<const char*>(node->getID()), DHT_ID_LENGTH);
+    WRITE_CHECK(fp, node->getID(), DHT_ID_LENGTH);
     // 4bytes reserved
-    o.write(zero, 4);
+    WRITE_CHECK(fp, zero, 4);
   }
-
-  o.flush();
-  if(!o) {
-    throw DL_ABORT_EX
-      (StringFormat("Failed to save DHT routing table. cause:%s",
-                    strerror(errno)).str());
+  if(fp.close() == EOF) {
+    throw DL_ABORT_EX(fmt("Failed to save DHT routing table to %s.",
+                          filename.c_str()));
   }
+  if(!File(filenameTemp).renameTo(filename)) {
+    throw DL_ABORT_EX(fmt("Failed to save DHT routing table to %s.",
+                          filename.c_str()));
+  }
+  A2_LOG_INFO("DHT routing table was saved successfully");
 }
 
 } // namespace aria2
