@@ -36,7 +36,6 @@
 
 #include <cstdlib>
 #include <cstring>
-#include <fstream>
 #include <sstream>
 #include <iostream>
 
@@ -56,6 +55,8 @@
 #include "error_code.h"
 #include "SimpleRandomizer.h"
 #include "bittorrent_helper.h"
+#include "BufferedFile.h"
+#include "console.h"
 #ifndef HAVE_DAEMON
 #include "daemon.h"
 #endif // !HAVE_DAEMON
@@ -75,17 +76,17 @@ void overrideWithEnv(Option& op, const OptionParser& optionParser,
     try {
       optionParser.findByName(pref)->parse(op, value);
     } catch(Exception& e) {
-      std::cerr << "Caught Error while parsing environment variable"
-                << " '" << envName << "'"
-                << "\n"
-                << e.stackTrace();
+      global::cerr()->printf
+        ("Caught Error while parsing environment variable '%s'\n%s\n",
+         envName.c_str(),
+         e.stackTrace().c_str());
     }
   }
 }
 } // namespace
 
 void option_processing(Option& op, std::vector<std::string>& uris,
-                       int argc, char* const argv[])
+                       int argc, char* argv[])
 {
   OptionParser oparser;
   oparser.setOptionHandlers(OptionHandlerFactory::createOptionHandlers());
@@ -133,27 +134,35 @@ void option_processing(Option& op, std::vector<std::string>& uris,
         ucfname;
 
       if(File(cfname).isFile()) {
-        std::ifstream cfstream(cfname.c_str(), std::ios::binary);
+        std::stringstream ss;
+        {
+          BufferedFile fp(cfname, BufferedFile::READ);
+          if(fp) {
+            fp.transfer(ss);
+          }
+        }
         try {
-          oparser.parse(op, cfstream);
+          oparser.parse(op, ss);
         } catch(OptionHandlerException& e) {
-          std::cerr << "Parse error in " << cfname << "\n"
-                    << e.stackTrace() << std::endl;
+          global::cerr()->printf("Parse error in %s\n%s\n",
+                                 cfname.c_str(),
+                                 e.stackTrace().c_str());
           SharedHandle<OptionHandler> h = oparser.findByName(e.getOptionName());
           if(h) {
-            std::cerr << "Usage:" << "\n"
-                      << oparser.findByName(e.getOptionName())->getDescription()
-                      << std::endl;
+            global::cerr()->printf
+              ("Usage:\n%s\n",
+               oparser.findByName(e.getOptionName())->getDescription().c_str());
           }
           exit(e.getErrorCode());
         } catch(Exception& e) {
-          std::cerr << "Parse error in " << cfname << "\n"
-                    << e.stackTrace() << std::endl;
+          global::cerr()->printf("Parse error in %s\n%s\n",
+                                 cfname.c_str(),
+                                 e.stackTrace().c_str());
           exit(e.getErrorCode());
         }
       } else if(!ucfname.empty()) {
-        std::cerr << fmt("Configuration file %s is not found.", cfname.c_str())
-                  << "\n";
+        global::cerr()->printf("Configuration file %s is not found.\n",
+                               cfname.c_str());
         showUsage(TAG_HELP, oparser);
         exit(error_code::UNKNOWN_ERROR);
       }
@@ -170,17 +179,25 @@ void option_processing(Option& op, std::vector<std::string>& uris,
     cmdstream.seekg(0, std::ios::beg);
     // finaly let's parse and store command-iine options.
     oparser.parse(op, cmdstream);
+#ifdef __MINGW32__
+    for(std::map<std::string, std::string>::iterator i = op.begin();
+        i != op.end(); ++i) {
+      if(!util::isUtf8((*i).second)) {
+        (*i).second = nativeToUtf8((*i).second);
+      }
+    }
+#endif // __MINGW32__
   } catch(OptionHandlerException& e) {
-    std::cerr << e.stackTrace() << "\n";
+    global::cerr()->printf("%s\n", e.stackTrace().c_str());
     SharedHandle<OptionHandler> h = oparser.findByName(e.getOptionName());
     if(h) {
-      std::cerr << "Usage:" << "\n"
-                << *h
-                << std::endl;
+      std::ostringstream ss;
+      ss << *h;
+      global::cerr()->printf("Usage:\n%s\n", ss.str().c_str());
     }
     exit(e.getErrorCode());
   } catch(Exception& e) {
-    std::cerr << e.stackTrace() << std::endl;
+    global::cerr()->printf("%s\n", e.stackTrace().c_str());
     showUsage(TAG_HELP, oparser);
     exit(e.getErrorCode());
   }
@@ -193,7 +210,7 @@ void option_processing(Option& op, std::vector<std::string>& uris,
 #endif // ENABLE_METALINK
      op.blank(PREF_INPUT_FILE)) {
     if(uris.empty()) {
-      std::cerr << MSG_URI_REQUIRED << std::endl;
+      global::cerr()->printf("%s\n", MSG_URI_REQUIRED);
       showUsage(TAG_HELP, oparser);
       exit(error_code::UNKNOWN_ERROR);
     }
